;;; DO NOT MODIFY THIS FILE
(if (featurep 'skk-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "skk/_pkg.el")

(package-provide 'skk :version 1.19 :type 'regular)

;;;***

;;;### (autoloads (skk-adjust-search-prog-list-for-auto-okuri skk-okuri-search) "skk-auto" "skk/skk-auto.el")

(defgroup skk-auto nil "SKK auto okuri process related customization." :prefix "skk-" :group 'skk)

(autoload 'skk-okuri-search "skk-auto" nil nil nil)

(autoload 'skk-adjust-search-prog-list-for-auto-okuri "skk-auto" nil nil nil)

;;;***

;;;### (autoloads (skk-previous-completion skk-completion skk-start-henkan-with-completion) "skk-comp" "skk/skk-comp.el")

(defgroup skk-comp nil "SKK completion related customization." :prefix "skk-" :group 'skk)

(autoload 'skk-start-henkan-with-completion "skk-comp" "\
▽モードで読みの補完を行なった後、変換する。
それ以外のモードではオリジナルのキーマップに割り付けられたコマンドをエミュレー
トする。" t nil)

(autoload 'skk-completion "skk-comp" nil nil nil)

(autoload 'skk-previous-completion "skk-comp" nil nil nil)

;;;***

;;;### (autoloads (skk-submit-bug-report) "skk-develop" "skk/skk-develop.el")

(autoload 'skk-submit-bug-report "skk-develop" "\
SKK のバグレポートを書くメールバッファを用意する。
mail-user-agent を設定することにより好みのメールインターフェイスを使用すること
ができる。例えば、Wanderlust を使用したい場合は下記のように設定する。

    (setq mail-user-agent 'wl-user-agent) " t nil)

;;;***

;;;### (autoloads nil "skk-foreword" "skk/skk-foreword.el")

(eval-and-compile (defconst skk-emacs-type (cond ((featurep 'xemacs) 'xemacs) ((and (boundp 'mule-version) (string< "4.0" mule-version) 'mule4)) ((and (boundp 'mule-version) (string< "3.0" mule-version) 'mule3)) ((and (boundp 'mule-version) (string< "2.0" mule-version) 'mule2)))))

;;;***

;;;### (autoloads (skk-henkan-face-off-and-remove-itself skk-ignore-dic-word skk-times skk-minus skk-plus skk-calc skk-gengo-to-ad skk-ad-to-gengo skk-clock skk-today skk-current-date) "skk-gadget" "skk/skk-gadget.el")

(defgroup skk-gadget nil "SKK gadget related customization." :prefix "skk-" :group 'skk)

(autoload 'skk-current-date "skk-gadget" nil nil nil)

(autoload 'skk-today "skk-gadget" "\
現在の日時を日本語表記で返す。
オプショナル引数の AND-TIME を指定すると、日付に時間を加える。
skk-date-ad と skk-number-style によって表示方法のカスタマイズが可能。" t nil)

(autoload 'skk-clock "skk-gadget" "\
デジタル時計をミニバッファに表示する。
quit するとその時点の日時を候補として挿入する。
quit したときに起動してからの経過時間をミニバッファに表示する。
interactive に起動する他、\"clock /(skk-clock)/\" などのエントリを SKK の辞書
に加え、\"/clock\"+ SPC で変換することによっても起動可。C-g で止まる。
実行変換で起動した場合は、C-g した時点の時点の日時を挿入する。
オプショナル引数の KAKUTEI-WHEN-QUIT が non-nil であれば C-g したときに確
定する。
オプショナル引数の TIME-SIGNAL が non-nil であれば、NTT の時報風に ding する。
それぞれ、\"clock /(skk-clock nil t)/\" のようなエントリを辞書に挿入すれば良い。
skk-date-ad と skk-number-style によって表示方法のカスタマイズが可能。" t nil)

(autoload 'skk-ad-to-gengo "skk-gadget" nil nil nil)

(autoload 'skk-gengo-to-ad "skk-gadget" nil nil nil)

(autoload 'skk-calc "skk-gadget" nil nil nil)

(autoload 'skk-plus "skk-gadget" nil nil nil)

(autoload 'skk-minus "skk-gadget" nil nil nil)

(autoload 'skk-times "skk-gadget" nil nil nil)

(autoload 'skk-ignore-dic-word "skk-gadget" nil nil nil)

(autoload 'skk-henkan-face-off-and-remove-itself "skk-gadget" nil nil nil)

;;;***

;;;### (autoloads (skk-isearch-mode-cleanup skk-isearch-mode-setup) "skk-isearch" "skk/skk-isearch.el")

(defgroup skk-isearch nil "SKK incremental search related customization." :prefix "skk-isearch-" :group 'skk)

(autoload 'skk-isearch-mode-setup "skk-isearch" "\
hook function called when skk isearch begin." nil nil)

(autoload 'skk-isearch-mode-cleanup "skk-isearch" "\
Hook function called when skk isearch is done." nil nil)

;;;***

;;;### (autoloads (skk-romaji-message skk-romaji-region skk-hurigana-katakana-message skk-hurigana-katakana-region skk-hurigana-message skk-hurigana-region skk-gyakubiki-katakana-message skk-gyakubiki-katakana-region skk-gyakubiki-message skk-gyakubiki-region) "skk-kakasi" "skk/skk-kakasi.el")

(defgroup skk-kakasi nil "SKK kakasi related customization." :prefix "skk-" :group 'skk)

(autoload 'skk-gyakubiki-region "skk-kakasi" "\
リージョンの漢字、送り仮名を全てひらがなに変換する。
オプショナル引数の ALL が non-nil ならば、複数の候補がある場合は、\"{}\" でく
くって表示する。
例えば、
    中島 -> {なかしま|なかじま}" t nil)

(autoload 'skk-gyakubiki-message "skk-kakasi" "\
リージョンの漢字、送り仮名を全てひらがなに変換後、エコーする。
オプショナル引数の ALL が non-nil ならば、複数の候補がある場合は、\"{}\" でく
くって表示する。
例えば、
    中島 -> {なかしま|なかじま}" t nil)

(autoload 'skk-gyakubiki-katakana-region "skk-kakasi" "\
リージョンの漢字、送り仮名を全てカタカナに変換する。
オプショナル引数の ALL が non-nil ならば、複数の候補がある場合は、\"{}\" でく
くって表示する。
例えば、
    中島 -> {ナカシマ|ナカジマ}" t nil)

(autoload 'skk-gyakubiki-katakana-message "skk-kakasi" "\
リージョンの漢字、送り仮名を全てカタカナに変換後、エコーする。
オプショナル引数の ALL が non-nil ならば、複数の候補がある場合は、\"{}\" でく
くって表示する。
例えば、
    中島 -> {ナカシマ|ナカジマ}" t nil)

(autoload 'skk-hurigana-region "skk-kakasi" "\
リージョンの漢字に全てふりがなを付ける。
例えば、
   \"変換前の漢字の脇に\" -> \"変換前[へんかんまえ]の漢字[かんじ]の脇[わき]に\"

オプショナル引数の ALL が non-nil ならば、複数の候補がある場合は、\"{}\" でく
くって表示する。
例えば、
    中島 -> {なかしま|なかじま}" t nil)

(autoload 'skk-hurigana-message "skk-kakasi" "\
リージョンの漢字に全てふりがなを付け、エコーする。
例えば、
   \"変換前の漢字の脇に\" -> \"変換前[へんかんまえ]の漢字[かんじ]の脇[わき]に\"

オプショナル引数の ALL が non-nil ならば、複数の候補がある場合は、\"{}\" でく
くって表示する。
例えば、
    中島 -> {なかしま|なかじま}" t nil)

(autoload 'skk-hurigana-katakana-region "skk-kakasi" "\
リージョンの漢字に全てフリガナを付ける。
例えば、
   \"変換前の漢字の脇に\" -> \"変換前[ヘンカンマエ]の漢字[カンジ]の脇[ワキ]に\"

オプショナル引数の ALL が non-nil ならば、複数の候補がある場合は、\"{}\" でく
くって表示する。
例えば、
    中島 -> {ナカシマ|ナカジマ}" t nil)

(autoload 'skk-hurigana-katakana-message "skk-kakasi" "\
リージョンの漢字に全てフリガナを付け、エコーする。
例えば、
   \"変換前の漢字の脇に\" -> \"変換前[ヘンカンマエ]の漢字[カンジ]の脇[ワキ]に\"

オプショナル引数の ALL が non-nil ならば、複数の候補がある場合は、\"{}\" でく
くって表示する。
例えば、
    中島 -> {ナカシマ|ナカジマ}" t nil)

(autoload 'skk-romaji-region "skk-kakasi" "\
リージョンの漢字、ひらがな、カタカナ、全英文字を全てローマ字に変換する。
変換には、ヘボン式を用いる。
例えば、
   \"漢字かな混じり文をローマ字に変換\"
    -> \"  kan'zi  kana  ma  ziri  bun'  woro-ma  zi ni hen'kan' \"

skk-romaji-*-by-hepburn が nil であれば、ローマ字への変換様式を訓令式に変更す
る。例えば、\"し\" はヘボン式では \"shi\" だが、訓令式では \"si\" となる。" t nil)

(autoload 'skk-romaji-message "skk-kakasi" "\
リージョンの漢字、ひらがな、カタカナ、全英文字を全てローマ字に変換し、エコーする。
変換には、ヘボン式を用いる。
例えば、
   \"漢字かな混じり文をローマ字に変換\"
    -> \"  kan'zi  kana  ma  ziri  bun'  woro-ma  zi ni hen'kan' \"

skk-romaji-*-by-hepburn が nil であれば、ローマ字への変換様式を訓令式に変更す
る。例えば、\"し\" はヘボン式では \"shi\" だが、訓令式では \"si\" となる。" t nil)

;;;***

;;;### (autoloads (skk-display-code-for-char-at-point skk-input-by-code-or-menu) "skk-kcode" "skk/skk-kcode.el")

(defgroup skk-kcode nil "SKK Kanji Code related customization." :prefix "skk-" :group 'skk)

(autoload 'skk-input-by-code-or-menu "skk-kcode" "\
7bit もしくは 8bit もしくは 区点コードに対応する 2byte 文字を挿入する。" t nil)

(autoload 'skk-display-code-for-char-at-point "skk-kcode" "\
ポイントにある文字の EUC コードと JIS コードを表示する。" t nil)

;;;***

;;;### (autoloads (skk-auto-fill-inactivate skk-inactivate skk-auto-fill-activate skk-activate) "skk-leim" "skk/skk-leim.el")

(autoload 'skk-activate "skk-leim" nil nil nil)

(autoload 'skk-auto-fill-activate "skk-leim" nil nil nil)

(autoload 'skk-inactivate "skk-leim" nil nil nil)

(autoload 'skk-auto-fill-inactivate "skk-leim" nil nil nil)

(register-input-method "japanese-skk" "Japanese" 'skk-activate nil "Simple Kana to Kanji conversion program")

(register-input-method "japanese-skk-auto-fill" "Japanese" 'skk-auto-fill-activate nil "Simple Kana to Kanji conversion program with auto-fill")

;;;***

;;;### (autoloads (skk-look-completion skk-look) "skk-look" "skk/skk-look.el")

(defgroup skk-look nil "SKK look conversion related customization." :prefix "skk-look-" :group 'skk)

(autoload 'skk-look "skk-look" nil nil nil)

(autoload 'skk-look-completion "skk-look" nil nil nil)

;;;***

;;;### (autoloads (skk-num skk-num-update-jisyo skk-num-henkan-key skk-num-initialize skk-num-process-user-minibuf-input skk-num-uniq) "skk-num" "skk/skk-num.el")

(defgroup skk-num nil "SKK number conversion related customization." :prefix "skk-num-" :group 'skk)

(autoload 'skk-num-uniq "skk-num" nil nil nil)

(autoload 'skk-num-process-user-minibuf-input "skk-num" nil nil nil)

(autoload 'skk-num-initialize "skk-num" nil nil nil)

(autoload 'skk-num-henkan-key "skk-num" nil nil nil)

(autoload 'skk-num-update-jisyo "skk-num" nil nil nil)

(autoload 'skk-num "skk-num" nil nil nil)

;;;***

;;;### (autoloads (skk-obsolete-put-obsolete-mark skk-obsolete-check-all-files skk-obsolete-check) "skk-obsolete" "skk/skk-obsolete.el")

(autoload 'skk-obsolete-check "skk-obsolete" "\
FILE 内の obsolete 変数名と obsolete 関数名をチェックし、書換える。" t nil)

(autoload 'skk-obsolete-check-all-files "skk-obsolete" "\
関連ファイル全ての obsolete 変数名と obsolete 関数名をチェックし、書換える。
C-u M-x skk-obsolete-check-all-files のように起動したときは、ディフォルトディレ
クトリにある SKK プログラムファイルもチェックを行なう。" t nil)

(autoload 'skk-obsolete-put-obsolete-mark "skk-obsolete" nil nil nil)

;;;***

;;;### (autoloads (skk-adjust-search-prog-list-for-server-search) "skk-server" "skk/skk-server.el")

(defgroup skk-server nil "SKK server related customization." :prefix "skk-server-" :group 'skk)

(autoload 'skk-adjust-search-prog-list-for-server-search "skk-server" nil nil nil)

;;;***

;;;### (autoloads (skk-tutorial) "skk-tut" "skk/skk-tut.el")

(defgroup skk-tut nil "SKK tutorial conversion related customization." :prefix "skk-tut-" :group 'skk)

(autoload 'skk-tutorial "skk-tut" "\
SKK チュートリアルを起動する。
C-u M-x skk-tutorial すると、チュートリアルファイルの選択が可能。" t nil)

;;;***

;;;### (autoloads (skk-viper-normalize-map) "skk-viper" "skk/skk-viper.el")

(defconst skk-viper-use-vip-prefix (not (fboundp 'viper-normalize-minor-mode-map-alist)))

(defconst skk-viper-normalize-map-function (if skk-viper-use-vip-prefix 'vip-normalize-minor-mode-map-alist 'viper-normalize-minor-mode-map-alist) "\
Viper が minor-mode-map-alist を調整するための関数。")

(autoload 'skk-viper-normalize-map "skk-viper" nil nil nil)

;;;***

;;;### (autoloads (skk-auto-fill-mode skk-mode skk-version) "skk" "skk/skk.el")

(autoload 'skk-version "skk" nil t nil)

(defgroup skk nil "SKK basic customization." :prefix "skk-" :group 'japanese :group 'input-method)

(autoload 'skk-mode "skk" "\
日本語入力モード。
マイナーモードの一種で、オリジナルのモードには影響を与えない。
負の引数を与えると SKK モードから抜ける。

An input mode for Japanese, converting romanized phonetic strings to kanji.

A minor mode, it should not affect the use of any major mode or
orthogonal minor modes.

In the initial SKK mode, hiragana submode, the mode line indicator is
\"かな\".  Lowercase romaji entry is automatically converted to
hiragana where possible.  The lowercase characters `q' and `l' change
submodes of SKK, and `x' is used as a prefix indicating a small kana.

`q' is used to toggle between hiragana and katakana (mode line
indicator \"カナ\") entry submodes.

`l' is used to enter ASCII submode (mode line indicator \"SKK\").
Uppercase `L' enters JISX0208 latin (wide ASCII) submode (mode line
indicator \"全英\").  `
' returns to hiragana submode from either
ASCII submode.

Kanji conversion is complex, but the basic principle is that the user
signals the appropriate stem to be matched against dictionary keys by
the use of uppercase letters.  Because SKK does not use grammatical
information, both the beginning and the end of the stem must be marked.

For non-inflected words (eg, nouns) consisting entirely of kanji, the
simplest way to invoke conversion is to enter the reading of the kanji,
the first character only in uppercase.  A leading \"▽\" indicates that
kanji conversion is in progress.  After entering the reading, press
space.  This invokes dictionary lookup, and the hiragana reading will be
redisplayed in kanji as the first candidate.  Pressing space again gives
the next candidate.  Further presses of space produce further candidates,
as well as a list of the next few candidates in the minibuffer.  Eg,
\"Benri\" => \"▽べんり\", and pressing space produces \"▼便利\" (the solid
triangle indicates that conversion is in progress).  Backspace steps
through the candidate list in reverse.

A candidate can be accepted by pressing `
', or by entering a
self-inserting character.  (Unlike other common Japanese input methods,
RET not only accepts the current candidate, but also inserts a line
break.)

Inflected words (verbs and adjectives), like non-inflected words, begin
entry with a capital letter.  However, for these words the end of the
kanji string is signaled by capitalizing the next mora.  Eg, \"TuyoI\"
=> \"▼強い\".  If no candidate is available at that point, the inflection
point will be indicated with an asterisk \"*\", and trailing characters
will be displayed until a candidate is recognized.  It will be
immediately displayed (pressing space is not necessary).  Space and
backspace are used to step forward and backward through the list of
candidates.

For more information, see the `skk' topic in Info.  (Japanese only.)

A tutorial is available in Japanese or English via \"M-x skk-tutorial\".
Use a prefix argument to choose the language.  The default is system-
dependent." t nil)

(autoload 'skk-auto-fill-mode "skk" "\
日本語入力モード。自動折り返し機能付き。
マイナーモードの一種で、オリジナルのモードには影響を与えない。
正の引数を与えると、強制的に auto-fill-mode 及び SKK モードに入る。
負の引数を与えると auto-fill-mode 及び SKK モードから抜ける。" t nil)

;;;***

;;;### (autoloads (Vip-mode vip-skk-mode) "vip" "skk/vip.el")

(autoload 'vip-skk-mode "vip" "\
Turn on both VIP-MODE and SKK-MODE. if ARG is nil, then toggle
SKK-MODE.  Then, change mode to insert mode." t nil)

(autoload 'Vip-mode "vip" "\
Turn on VIP emulation of VI." t nil)

;;;***

(provide 'skk-autoloads)
